package org.objectweb.petals.component.common;

import javax.jbi.JBIException;
import javax.jbi.component.Bootstrap;
import javax.jbi.component.InstallationContext;
import javax.management.ObjectName;

import org.objectweb.petals.component.common.util.PetalsExtensionsUtil;

/**
 * A simple bootstrap implementation. This bootstrap simplies copy the JBI
 * extensions in a properties file that will be used by components.
 *
 * @author ofabre chamerling - eBM WebSourcing
 *
 */
public class SimpleBootstrap implements Bootstrap {

    protected InstallationContext installContext;

    protected ComponentPropertiesManager propertiesManager;

    /*
     * (non-Javadoc)
     *
     * @see javax.jbi.component.Bootstrap#cleanUp()
     */
    public void cleanUp() throws JBIException {
    }

    /**
     * Returns null by default as this class does not provides JMX extension
     * MBean
     *
     * @see javax.jbi.component.ComponentLifeCycle#getExtensionMBeanName()
     */
    public ObjectName getExtensionMBeanName() {
        return null;
    }

    /*
     * (non-Javadoc)
     *
     * @see javax.jbi.component.Bootstrap#init(javax.jbi.component.InstallationContext)
     */
    public void init(InstallationContext installContext) throws JBIException {
        this.installContext = installContext;
        try {
            doInit(installContext);
        } catch (Exception e) {
            throw new JBIException("Initialization exception", e);
        }
    }

    /**
     * The component properties are loaded in this order :
     * <ol>
     * <li>Get the default values defined in the CDK (thread pool, ...)</li>
     * <li>Get values from the JBI extensions (can override the default ones)</li>
     * <li>Get additional properties defined in the component</li>
     * </ol>
     *
     * @param installContext
     */
    private void doInit(InstallationContext installContext) {
        if (this.propertiesManager == null) {
            this.propertiesManager = new ComponentPropertiesManager();
            this.propertiesManager.setRootPath(installContext.getInstallRoot());

            // get the default values
            this.propertiesManager.initDefaultValues();

            // get the properties from the jbi extensions. Will override the
            // default ones
            this.propertiesManager.setAttributes(PetalsExtensionsUtil
                .getExtensionsAsProperties(installContext));
        }

        // do additional configuration
        doConfig();

        // last step, persist properties
        this.propertiesManager.save();
    }

    /**
     * Do additional configuration during the bootstrap initialization phase.
     * This method is called at the end of the initialization and can be
     * overridden to provide specific additional configuration.
     *
     */
    protected void doConfig() {
    }

    /**
     *
     * @return
     */
    protected ComponentPropertiesManager getComponentPropertiesManager() {
        return this.propertiesManager;
    }

    /*
     * (non-Javadoc)
     *
     * @see javax.jbi.component.Bootstrap#onInstall()
     */
    public void onInstall() throws JBIException {
    }

    /*
     * (non-Javadoc)
     *
     * @see javax.jbi.component.Bootstrap#onUninstall()
     */
    public void onUninstall() throws JBIException {
    }
}
